@@ -30,8 +30,11 @@ from calibre.utils.filenames import (
     is_case_sensitive, samefile, hardlink_file, ascii_filename,
     WindowsAtomicFolderMove, atomic_rename, remove_dir_if_empty,
     copytree_using_links, copyfile_using_links)
-from calibre.utils.magick.draw import save_cover_data_to
-from calibre.utils.formatter_functions import load_user_template_functions
+from calibre.utils.img import save_cover_data_to
+from calibre.utils.formatter_functions import (load_user_template_functions,
+            unload_user_template_functions,
+            compile_user_template_functions,
+            formatter_functions)
 from calibre.db.tables import (OneToOneTable, ManyToOneTable, ManyToManyTable,
         SizeTable, FormatsTable, AuthorsTable, IdentifiersTable, PathTable,
         CompositeTable, UUIDTable, RatingTable)
@@ -47,6 +50,7 @@ Differences in semantics from pysqlite:
 '''
 CUSTOM_DATA_TYPES = frozenset(['rating', 'text', 'comments', 'datetime',
     'int', 'float', 'bool', 'series', 'composite', 'enumeration'])
+WINDOWS_RESERVED_NAMES = frozenset('CON PRN AUX NUL COM1 COM2 COM3 COM4 COM5 COM6 COM7 COM8 COM9 LPT1 LPT2 LPT3 LPT4 LPT5 LPT6 LPT7 LPT8 LPT9'.split())
 
 
 class DynamicFilter(object):  # {{{
@@ -64,6 +68,7 @@ class DynamicFilter(object):  # {{{
         self.ids = frozenset(ids)
 # }}}
 
+
 class DBPrefs(dict):  # {{{
 
     'Store preferences as key:value pairs in the db'
@@ -159,6 +164,8 @@ class DBPrefs(dict):  # {{{
 # }}}
 
 # Extra collators {{{
+
+
 def pynocase(one, two, encoding='utf-8'):
     if isbytestring(one):
         try:
@@ -172,11 +179,13 @@ def pynocase(one, two, encoding='utf-8'):
             pass
     return cmp(one.lower(), two.lower())
 
+
 def _author_to_author_sort(x):
     if not x:
         return ''
     return author_to_author_sort(x.replace('|', ','))
 
+
 def icu_collator(s1, s2):
     return cmp(sort_key(force_unicode(s1, 'utf-8')),
                sort_key(force_unicode(s2, 'utf-8')))
@@ -184,6 +193,8 @@ def icu_collator(s1, s2):
 # }}}
 
 # Unused aggregators {{{
+
+
 def Concatenate(sep=','):
     '''String concatenation aggregator for sqlite'''
 
@@ -198,6 +209,7 @@ def Concatenate(sep=','):
 
     return ([], step, finalize)
 
+
 def SortedConcatenate(sep=','):
     '''String concatenation aggregator for sqlite, sorted by supplied index'''
 
@@ -212,6 +224,7 @@ def SortedConcatenate(sep=','):
 
     return ({}, step, finalize)
 
+
 def IdentifiersConcat():
     '''String concatenation aggregator for the identifiers map'''
 
@@ -223,6 +236,7 @@ def IdentifiersConcat():
 
     return ([], step, finalize)
 
+
 def AumSortedConcatenate():
     '''String concatenation aggregator for the author sort map'''
 
@@ -243,6 +257,7 @@ def AumSortedConcatenate():
 
 # }}}
 
+
 class Connection(apsw.Connection):  # {{{
 
     BUSY_TIMEOUT = 10000  # milliseconds
@@ -251,7 +266,7 @@ class Connection(apsw.Connection):  # {{{
         apsw.Connection.__init__(self, path)
 
         self.setbusytimeout(self.BUSY_TIMEOUT)
-        self.execute('pragma cache_size=5000')
+        self.execute('pragma cache_size=-5000')
         self.execute('pragma temp_store=2')
 
         encoding = self.execute('pragma encoding').next()[0]
@@ -303,6 +318,12 @@ class Connection(apsw.Connection):  # {{{
 
 # }}}
 
+
+def set_global_state(backend):
+    load_user_template_functions(
+        backend.library_id, (), precompiled_user_functions=backend.get_user_template_functions())
+
+
 class DB(object):
 
     PATH_LIMIT = 40 if iswindows else 100
@@ -388,9 +409,21 @@ class DB(object):
         self.initialize_prefs(default_prefs, restore_all_prefs, progress_callback)
         self.initialize_custom_columns()
         self.initialize_tables()
+        self.set_user_template_functions(compile_user_template_functions(
+                                 self.prefs.get('user_template_functions', [])))
         if load_user_formatter_functions:
-            load_user_template_functions(self.library_id,
-                                        self.prefs.get('user_template_functions', []))
+            set_global_state(self)
+
+    def get_template_functions(self):
+        return self._template_functions
+
+    def get_user_template_functions(self):
+        return self._user_template_functions
+
+    def set_user_template_functions(self, user_formatter_functions):
+        self._user_template_functions = user_formatter_functions
+        self._template_functions = formatter_functions().get_builtins().copy()
+        self._template_functions.update(user_formatter_functions)
 
     def initialize_prefs(self, default_prefs, restore_all_prefs, progress_callback):  # {{{
         self.prefs = DBPrefs(self)
@@ -431,19 +464,21 @@ class DB(object):
         defs['similar_series_search_key'] = 'series'
         defs['similar_series_match_kind'] = 'match_any'
         defs['book_display_fields'] = [
-        ('title', False), ('authors', True), ('formats', True),
-        ('series', True), ('identifiers', True), ('tags', True),
+        ('title', False), ('authors', True), ('series', True),
+        ('identifiers', True), ('tags', True), ('formats', True),
         ('path', True), ('publisher', False), ('rating', False),
         ('author_sort', False), ('sort', False), ('timestamp', False),
         ('uuid', False), ('comments', True), ('id', False), ('pubdate', False),
         ('last_modified', False), ('size', False), ('languages', False),
         ]
+        defs['qv_display_fields'] = [('title', True), ('authors', True), ('series', True)]
         defs['virtual_libraries'] = {}
         defs['virtual_lib_on_startup'] = defs['cs_virtual_lib_on_startup'] = ''
         defs['virt_libs_hidden'] = defs['virt_libs_order'] = ()
         defs['update_all_last_mod_dates_on_start'] = False
         defs['field_under_covers_in_grid'] = 'title'
         defs['cover_browser_title_template'] = '{title}'
+        defs['cover_browser_subtitle_field'] = 'rating'
 
         # Migrate the bool tristate tweak
         defs['bools_are_tristate'] = \
@@ -1029,13 +1064,18 @@ class DB(object):
         data = self.custom_field_metadata(label, num)
         self.execute('UPDATE custom_columns SET mark_for_delete=1 WHERE id=?', (data['num'],))
 
-    def close(self, force=False):
+    def close(self, force=False, unload_formatter_functions=True):
         if getattr(self, '_conn', None) is not None:
+            if unload_formatter_functions:
+                try:
+                    unload_user_template_functions(self.library_id)
+                except Exception:
+                    pass
             self._conn.close(force)
             del self._conn
 
     def reopen(self, force=False):
-        self.close(force)
+        self.close(force=force, unload_formatter_functions=False)
         self._conn = None
         self.conn
 
@@ -1064,7 +1104,7 @@ class DB(object):
                     shell.process_command('.read ' + fname.replace(os.sep, '/'))
                     conn.execute('PRAGMA user_version=%d;'%uv)
 
-                self.close()
+                self.close(unload_formatter_functions=False)
                 try:
                     atomic_rename(tmpdb, self.dbpath)
                 finally:
@@ -1139,6 +1179,8 @@ class DB(object):
         if not author:
             author = ascii_filename(_('Unknown')).decode(
                     'ascii', 'replace')
+        if author.upper() in WINDOWS_RESERVED_NAMES:
+            author += 'w'
         return '%s/%s%s' % (author, title, book_id)
 
     def construct_file_name(self, book_id, title, author, extlen):
@@ -1373,7 +1415,7 @@ class DB(object):
                     os.remove(path)
         else:
             if no_processing:
-                with open(path, 'wb') as f:
+                with lopen(path, 'wb') as f:
                     f.write(data)
             else:
                 try:
@@ -1617,6 +1659,7 @@ class DB(object):
 
     def get_custom_book_data(self, name, book_ids, default=None):
         book_ids = frozenset(book_ids)
+
         def safe_load(val):
             try:
                 return json.loads(val, object_hook=from_json)
