@@ -8,6 +8,7 @@ __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import cPickle, os, sys
 from collections import defaultdict, OrderedDict
+from itertools import chain
 from threading import Thread
 from functools import partial
 
@@ -27,6 +28,7 @@ from calibre.gui2.languages import LanguagesEdit
 from calibre.gui2.progress_indicator import ProgressIndicator
 from calibre.gui2.tweak_book import dictionaries, current_container, set_book_locale, tprefs, editors
 from calibre.gui2.tweak_book.widgets import Dialog
+from calibre.gui2.widgets2 import FlowLayout
 from calibre.spell import DictionaryLocale
 from calibre.spell.dictionary import (
     builtin_dictionaries, custom_dictionaries, best_locale_for_language,
@@ -42,12 +44,14 @@ DICTIONARY = 2
 
 _country_map = None
 
+
 def country_map():
     global _country_map
     if _country_map is None:
         _country_map = cPickle.loads(P('localization/iso3166.pickle', data=True, allow_user_override=False))
     return _country_map
 
+
 class AddDictionary(QDialog):  # {{{
 
     def __init__(self, parent=None):
@@ -127,6 +131,7 @@ class AddDictionary(QDialog):  # {{{
 
 # User Dictionaries {{{
 
+
 class UserWordList(QListWidget):
 
     def __init__(self, parent=None):
@@ -587,6 +592,8 @@ class ManageDictionaries(Dialog):  # {{{
 # }}}
 
 # Spell Check Dialog {{{
+
+
 class WordsModel(QAbstractTableModel):
 
     word_ignored = pyqtSignal(object, object)
@@ -669,6 +676,7 @@ class WordsModel(QAbstractTableModel):
     def sort_key(self, col):
         if col == 0:
             f = (lambda x: x) if tprefs['spell_check_case_sensitive_sort'] else primary_sort_key
+
             def key(w):
                 return f(w[0])
         elif col == 1:
@@ -752,7 +760,10 @@ class WordsModel(QAbstractTableModel):
     def replace_word(self, w, new_word):
         # Hack to deal with replacement words that are actually multiple words,
         # ignore all words except the first
-        new_word = split_into_words(new_word)[0]
+        try:
+            new_word = split_into_words(new_word)[0]
+        except IndexError:
+            new_word = ''
         for location in self.words[w]:
             location.replace(new_word)
         if w[0] == new_word:
@@ -801,11 +812,13 @@ class WordsModel(QAbstractTableModel):
         except ValueError:
             return -1
 
+
 class WordsView(QTableView):
 
     ignore_all = pyqtSignal()
     add_all = pyqtSignal(object)
     change_to = pyqtSignal(object, object)
+    current_changed = pyqtSignal(object, object)
 
     def __init__(self, parent=None):
         QTableView.__init__(self, parent)
@@ -819,11 +832,11 @@ class WordsView(QTableView):
             self.copy_to_clipboard()
             ev.accept()
             return
+        before = self.currentIndex()
         ret = QTableView.keyPressEvent(self, ev)
-        if ev.key() in (Qt.Key_PageUp, Qt.Key_PageDown, Qt.Key_Up, Qt.Key_Down):
-            idx = self.currentIndex()
-            if idx.isValid():
-                self.scrollTo(idx)
+        after = self.currentIndex()
+        if after.row() != before.row() and after.isValid():
+            self.scrollTo(after)
         return ret
 
     def highlight_row(self, row):
@@ -845,8 +858,8 @@ class WordsView(QTableView):
             for s in dictionaries.suggestions(*w):
                 cm.addAction(s, partial(self.change_to.emit, w, s))
 
-        m.addAction(_('Ignore/Unignore all selected words'), self.ignore_all)
-        a = m.addAction(_('Add/Remove all selected words'))
+        m.addAction(_('Ignore/un-ignore all selected words'), self.ignore_all)
+        a = m.addAction(_('Add/remove all selected words'))
         am = QMenu()
         a.setMenu(am)
         for dic in sorted(dictionaries.active_user_dictionaries, key=lambda x:sort_key(x.name)):
@@ -864,6 +877,14 @@ class WordsView(QTableView):
         if words:
             QApplication.clipboard().setText('\n'.join(words))
 
+    def currentChanged(self, cur, prev):
+        self.current_changed.emit(cur, prev)
+
+    @property
+    def current_word(self):
+        return self.model().word_for_row(self.currentIndex().row())
+
+
 class SpellCheck(Dialog):
 
     work_finished = pyqtSignal(object, object, object)
@@ -937,7 +958,7 @@ class SpellCheck(Dialog):
         w.add_all.connect(self.add_all)
         w.activated.connect(self.word_activated)
         w.change_to.connect(self.change_to)
-        w.currentChanged = self.current_word_changed
+        w.current_changed.connect(self.current_word_changed)
         state = tprefs.get(self.state_name, None)
         hh = self.words_view.horizontalHeader()
         h.addWidget(w)
@@ -1008,9 +1029,9 @@ class SpellCheck(Dialog):
         cs2.setToolTip(_('When filtering the list of words, be case sensitive'))
         cs2.setChecked(tprefs['spell_check_case_sensitive_search'])
         cs2.stateChanged.connect(self.search_type_changed)
-        self.hb = h = QHBoxLayout()
+        self.hb = h = FlowLayout()
         self.summary = s = QLabel('')
-        self.main.l.addLayout(h), h.addWidget(s), h.addWidget(om), h.addWidget(cs), h.addWidget(cs2), h.addStretch(1)
+        self.main.l.addLayout(h), h.addWidget(s), h.addWidget(om), h.addWidget(cs), h.addWidget(cs2)
 
     def keyPressEvent(self, ev):
         if ev.key() in (Qt.Key_Enter, Qt.Key_Return):
@@ -1072,11 +1093,19 @@ class SpellCheck(Dialog):
                     in_user_dictionary = dictionaries.word_in_user_dictionary(*w)
             suggestions = dictionaries.suggestions(*w)
             self.suggested_list.clear()
-            for i, s in enumerate(suggestions):
+            word_suggested = False
+            seen = set()
+            for i, s in enumerate(chain(suggestions, (current_word,))):
+                if s in seen:
+                    continue
+                seen.add(s)
                 item = QListWidgetItem(s, self.suggested_list)
                 if i == 0:
                     self.suggested_list.setCurrentItem(item)
                     self.suggested_word.setText(s)
+                    word_suggested = True
+            if not word_suggested:
+                self.suggested_word.setText(current_word)
 
         prefix = b.unign_text if ignored else b.ign_text
         b.setText(prefix + ' ' + current_word)
@@ -1122,6 +1151,8 @@ class SpellCheck(Dialog):
             self.word_replaced.emit(changed_files)
             w = self.words_model.replace_word(w, new_word)
             row = self.words_model.row_for_word(w)
+            if row == -1:
+                row = self.words_view.currentIndex().row()
             if row > -1:
                 self.words_view.highlight_row(row)
 
@@ -1217,6 +1248,8 @@ class SpellCheck(Dialog):
         self.words_model.clear()
 
     def work_done(self, words, spell_map, change_request):
+        row = self.words_view.rowAt(5)
+        before_word = self.words_view.current_word
         self.end_work()
         if not isinstance(words, dict):
             return error_dialog(self, _('Failed to check spelling'), _(
@@ -1225,15 +1258,20 @@ class SpellCheck(Dialog):
         if not self.isVisible():
             return
         self.words_model.set_data(words, spell_map)
+        wrow = self.words_model.row_for_word(before_word)
+        if 0 <= wrow < self.words_model.rowCount():
+            row = wrow
+        if row < 0 or row >= self.words_model.rowCount():
+            row = 0
         col, reverse = self.words_model.sort_on
         self.words_view.horizontalHeader().setSortIndicator(
             col, Qt.DescendingOrder if reverse else Qt.AscendingOrder)
-        self.words_view.highlight_row(0)
         self.update_summary()
         self.initialize_user_dictionaries()
         if self.words_model.rowCount() > 0:
             self.words_view.resizeRowToContents(0)
             self.words_view.verticalHeader().setDefaultSectionSize(self.words_view.rowHeight(0))
+        self.words_view.highlight_row(row)
         if change_request is not None:
             w, new_word = change_request
             if w in self.words_model.words:
@@ -1274,6 +1312,8 @@ class SpellCheck(Dialog):
 # }}}
 
 # Find next occurrence  {{{
+
+
 def find_next(word, locations, current_editor, current_editor_name,
               gui_parent, show_editor, edit_file):
     files = OrderedDict()
@@ -1307,6 +1347,7 @@ def find_next(word, locations, current_editor, current_editor_name,
             return True
     return False
 
+
 def find_next_error(current_editor, current_editor_name, gui_parent, show_editor, edit_file):
     files = get_checkable_file_names(current_container())[0]
     if current_editor_name not in files:
@@ -1332,6 +1373,7 @@ def find_next_error(current_editor, current_editor_name, gui_parent, show_editor
 
 # }}}
 
+
 if __name__ == '__main__':
     app = QApplication([])
     dictionaries.initialize()
