@@ -12,14 +12,10 @@ from unittest import skipIf
 from glob import glob
 from threading import Event
 
-try:
-    from calibre.utils.certgen import create_server_cert
-except ImportError:
-    create_server_cert = None
-
 from calibre.srv.pre_activated import has_preactivated_support
 from calibre.srv.tests.base import BaseTest, TestServer
 from calibre.ptempfile import TemporaryDirectory
+from calibre.utils.certgen import create_server_cert
 from calibre.utils.monotonic import monotonic
 is_ci = os.environ.get('CI', '').lower() == 'true'
 
@@ -88,7 +84,7 @@ class LoopTest(BaseTest):
             self.ae(0, sum(int(w.is_alive()) for w in server.loop.pool.workers))
         # Test shutdown with hung worker
         block = Event()
-        with TestServer(lambda data:block.wait(), worker_count=3, shutdown_timeout=0.01, timeout=0.01) as server:
+        with TestServer(lambda data:block.wait(), worker_count=3, shutdown_timeout=0.1, timeout=0.01) as server:
             pool = server.loop.pool
             self.ae(3, sum(int(w.is_alive()) for w in pool.workers))
             conn = server.connect()
@@ -187,7 +183,6 @@ class LoopTest(BaseTest):
         set(b'123456\n7', 4, 2, READ)
         self.ae(buf.readline(), b'56\n')
 
-    @skipIf(create_server_cert is None, 'certgen module not available')
     def test_ssl(self):
         'Test serving over SSL'
         address = '127.0.0.1'
@@ -288,6 +283,7 @@ class LoopTest(BaseTest):
         status, result, tb, was_aborted = jm.job_status(job_id)
         self.assertEqual(status, 'finished')
         self.assertFalse(was_aborted)
-        self.assertTrue(tb), self.assertIn('a testing error', tb)
+        self.assertTrue(tb)
+        self.assertIn('a testing error', tb)
         jm.start_job('simple test', 'calibre.srv.jobs', 'sleep_test', args=(1.0,))
         jm.shutdown(), jm.wait_for_shutdown(monotonic() + 1)
