@@ -60,6 +60,18 @@ def get_system_locale():
     return lang
 
 
+def sanitize_lang(lang):
+    if lang:
+        match = re.match('[a-z]{2,3}(_[A-Z]{2}){0,1}', lang)
+        if match:
+            lang = match.group()
+    if lang == 'zh':
+        lang = 'zh_CN'
+    if not lang:
+        lang = 'en'
+    return lang
+
+
 def get_lang():
     'Try to figure out what language to display the interface in'
     from calibre.utils.config_base import prefs
@@ -73,15 +85,7 @@ def get_lang():
         import traceback
         traceback.print_exc()
         lang = None
-    if lang:
-        match = re.match('[a-z]{2,3}(_[A-Z]{2}){0,1}', lang)
-        if match:
-            lang = match.group()
-    if lang == 'zh':
-        lang = 'zh_CN'
-    if not lang:
-        lang = 'en'
-    return lang
+    return sanitize_lang(lang)
 
 
 def is_rtl():
@@ -118,13 +122,19 @@ def get_all_translators():
                 yield lang, GNUTranslations(buf)
 
 
-def get_single_translator(mpath):
+def get_single_translator(mpath, which='messages'):
     from zipfile import ZipFile
     with ZipFile(P('localization/locales.zip', allow_user_override=False), 'r') as zf:
-        buf = cStringIO.StringIO(zf.read(mpath + '/messages.mo'))
+        buf = cStringIO.StringIO(zf.read(mpath + '/%s.mo' % which))
         return GNUTranslations(buf)
 
 
+def get_iso639_translator(lang):
+    lang = sanitize_lang(lang)
+    mpath = get_lc_messages_path(lang) if lang else None
+    return get_single_translator(mpath, 'iso639') if mpath else None
+
+
 def get_translator(bcp_47_code):
     parts = bcp_47_code.replace('-', '_').split('_')[:2]
     parts[0] = lang_as_iso639_1(parts[0].lower()) or 'en'
@@ -218,6 +228,10 @@ def set_translators():
     if t is None:
         t = NullTranslations()
 
+    try:
+        set_translators.lang = t.info().get('language')
+    except Exception:
+        pass
     t.install(unicode=True, names=('ngettext',))
     # Now that we have installed a translator, we have to retranslate the help
     # for the global prefs object as it was instantiated in get_lang(), before
@@ -226,6 +240,9 @@ def set_translators():
     prefs.retranslate_help()
 
 
+set_translators.lang = None
+
+
 _iso639 = None
 _extra_lang_codes = {
         'pt_BR' : _('Brazilian Portuguese'),
@@ -310,6 +327,8 @@ if False:
     _('Fonts')
     _('&Step up')
     _('Step &down')
+    _('Close without Saving')
+    _('Close Tab')
 
 _lcase_map = {}
 for k in _extra_lang_codes:
@@ -325,13 +344,7 @@ def _load_iso639():
     return _iso639
 
 
-def get_language(lang):
-    translate = _
-    lang = _lcase_map.get(lang, lang)
-    if lang in _extra_lang_codes:
-        # The translator was not active when _extra_lang_codes was defined, so
-        # re-translate
-        return translate(_extra_lang_codes[lang])
+def get_iso_language(lang_trans, lang):
     iso639 = _load_iso639()
     ans = lang
     lang = lang.split('_')[0].lower()
@@ -342,10 +355,17 @@ def get_language(lang):
             ans = iso639['by_3b'][lang]
         else:
             ans = iso639['by_3t'].get(lang, ans)
-    try:
-        return _lang_trans.ugettext(ans)
-    except AttributeError:
-        return translate(ans)
+    return lang_trans(ans)
+
+
+def get_language(lang):
+    translate = _
+    lang = _lcase_map.get(lang, lang)
+    if lang in _extra_lang_codes:
+        # The translator was not active when _extra_lang_codes was defined, so
+        # re-translate
+        return translate(_extra_lang_codes[lang])
+    return get_iso_language(getattr(_lang_trans, 'ugettext', translate), lang)
 
 
 def calibre_langcode_to_name(lc, localize=True):
@@ -439,15 +459,23 @@ def get_udc():
     return _udc
 
 
+def user_manual_stats():
+    stats = getattr(user_manual_stats, 'stats', None)
+    if stats is None:
+        import json
+        try:
+            stats = json.loads(P('user-manual-translation-stats.json', allow_user_override=False, data=True))
+        except EnvironmentError:
+            stats = {}
+        user_manual_stats.stats = stats
+    return stats
+
+
 def localize_user_manual_link(url):
     lc = lang_as_iso639_1(get_lang())
     if lc == 'en':
         return url
-    import json
-    try:
-        stats = json.loads(P('user-manual-translation-stats.json', allow_user_override=False, data=True))
-    except EnvironmentError:
-        return url
+    stats = user_manual_stats()
     if stats.get(lc, 0) < 0.3:
         return url
     from urlparse import urlparse, urlunparse
@@ -457,3 +485,27 @@ def localize_user_manual_link(url):
     parts = list(parts)
     parts[2] = path
     return urlunparse(parts)
+
+
+def website_languages():
+    stats = getattr(website_languages, 'stats', None)
+    if stats is None:
+        try:
+            stats = frozenset(P('localization/website-languages.txt', allow_user_override=False, data=True).split())
+        except EnvironmentError:
+            stats = frozenset()
+        website_languages.stats = stats
+    return stats
+
+
+def localize_website_link(url):
+    lc = lang_as_iso639_1(get_lang())
+    langs = website_languages()
+    if lc == 'en' or lc not in langs:
+        return url
+    from urlparse import urlparse, urlunparse
+    parts = urlparse(url)
+    path = '/{}{}'.format(lc, parts.path)
+    parts = list(parts)
+    parts[2] = path
+    return urlunparse(parts)
